/* 
    LCFMETER - Firmware for measure L, C, F using STM32F100 device
    Copyright (C) 2012  Lucjan Bryndza <lucck@boff.pl>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>. 
*/

/*----------------------------------------------------------*/
/*
 * fixed_queue.hpp
 *
 *  Created on: 2009-12-09
 *      Author: lucck
 */
/*----------------------------------------------------------*/
#ifndef FIXED_QUEUE_HPP_
#define FIXED_QUEUE_HPP_
/*----------------------------------------------------------*/
#include <cstddef>

/*----------------------------------------------------------*/
namespace fnd
{

/*----------------------------------------------------------*/
template <typename T, std::size_t Sz=128>
class fixed_queue
{
public:

	//Fixed queue constructor
	fixed_queue():head(0),tail(0) {};

	//Push element
	bool push(const T &c)
	{
		std::size_t next;
		// check if FIFO has room
		next = (head + 1) % Sz;
		if (next == tail) return true;
		buf[head] = c;
		head = next;
		return false;
	}

	T& front() { return buf[tail]; }
	const T& front() const { return buf[tail]; }

	//Pop element
	bool pop(T &c)
	{
		std::size_t next;
		// check if FIFO has data
		if (head == tail) return true;
		next = (tail + 1) % Sz;
		c = buf[tail];
		tail = next;
		return false;
	}

	//Get size
	std::size_t size() const
	{
		return (Sz + head - tail) % Sz;
	}

	//Get free count
	std::size_t free() const
	{
		return (Sz - 1 - size());
	}

	//Clear the fifo context
	void erase()
	{
		head = 0;
		tail = 0;
	}

private:
	T buf[Sz];
	std::size_t head;
	std::size_t tail;
};

/*----------------------------------------------------------*/

}
/*----------------------------------------------------------*/
#endif /* FIXED_QUEUE_HPP_ */
