/* 
    LCFMETER - Firmware for measure L, C, F using STM32F100 device
    Copyright (C) 2012  Lucjan Bryndza <lucck@boff.pl>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>. 
*/

/*
 * hd44xx_bus.hpp
 *
 *  Created on: 08-01-2012
 *      Author: lucck
 */
/*----------------------------------------------------------*/
#ifndef HD44XX_DISPLAY_HPP_
#define HD44XX_DISPLAY_HPP_
/*----------------------------------------------------------*/
#include <stdint.h>
/*----------------------------------------------------------*/
namespace fnd
{
namespace lcd
{
	class hd44xx_display
	{
	public:
		enum err
		{
			EDISPLAY_OK,
			EDISPLAY_TIMEOUT=-675776,
			EDISPLAY_INVALID_PARAM
		};
		hd44xx_display();
		virtual ~hd44xx_display() {}
		int clear();
		int cursor(bool enable)
		{
			write_dev(CMD_DISP_ON|(enable?0x07:0x04),W_INSTR);
			return wait4dev();
		}
		int putc(char ch)
		{
			write_dev(ch,W_DATA);
			return wait4dev();
		}
		int erase(int n_chars)
		{
			int ret;
			for(int i=0; i<n_chars; i++)
				if((ret = putc(' ')) != 0) return ret;
			return 0;
		}
		int setpos(int x, int y);
		int read_currpos();
		int show_icon(char chpos, const char *str);
        int progress_bar(int x, int y, int value, int width, char chpos=0);
	protected:
		virtual void write8_i(uint8_t addr, uint8_t value ) = 0;
		virtual void write8_n(uint8_t addr, uint8_t value ) = 0;
		virtual uint8_t read8( uint8_t addr )  = 0;
		virtual void mdelay( unsigned period)  = 0;
	private:
		int wait4dev();
		enum wtype { W_INSTR, W_DATA };
		void write_dev(unsigned char val, wtype command)
		{
			if( !m_initialized )
			{
				init_dev();
			}
			write8_n(command, val);
		}
		uint8_t read_dev( wtype command )
		{
			if( !m_initialized )
			{
				init_dev();
			}
			return read8( command );
		}
		int init_dev();
	private:
		static const unsigned CMD_DISP_ON = 8;
		bool m_initialized;
	private: 	//Noncopyable
		hd44xx_display(hd44xx_display &);
		hd44xx_display& operator=(const hd44xx_display&);
	};
/*----------------------------------------------------------*/
	class lfmt
	{
		friend hd44xx_display& operator<<(hd44xx_display &o,const lfmt &fmt);
	public:
		enum
		{
			dec = 10,
			hex = 16
		};
		lfmt(unsigned val_, int fmt_, char fmtch_='0', short base_=dec)
		: val(val_),fmt(fmt_),fmtch(fmtch_) , base(base_) {}
	private:
		unsigned val;
		int fmt;
		char fmtch;
		short base;
	};
/* ------------------------------------------------------------------ */
	struct icon
	{
		friend hd44xx_display& operator<<(hd44xx_display &o,const icon &fmt);
	protected:
		virtual const char* get_data() const = 0;
		virtual char get_char() const = 0;
	};
/* ------------------------------------------------------------------ */
	class pos
	{
		friend hd44xx_display& operator<<(hd44xx_display &o,const pos &fmt);
	public:
		pos(int x_, int y_)
		:x(x_), y(y_) {}
	private:
		unsigned char x;
		unsigned char y;
	};
/* ------------------------------------------------------------------ */
	hd44xx_display& operator<<(hd44xx_display &o,const char *str);

	hd44xx_display& operator<<(hd44xx_display &o,unsigned value);

	hd44xx_display& operator<<(hd44xx_display &o,const lfmt &fmt);

	hd44xx_display& operator<<(hd44xx_display &o,const icon &fmt);

	hd44xx_display& operator<<(hd44xx_display &o,const pos &fmt);

	hd44xx_display& operator<<(hd44xx_display &o,int value);
/* ------------------------------------------------------------------ */

}}
/*----------------------------------------------------------*/
#endif /* HD44XX_BUS_HPP_ */
