/* 
    LCFMETER - Firmware for measure L, C, F using STM32F100 device
    Copyright (C) 2012  Lucjan Bryndza <lucck@boff.pl>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>. 
*/

/*
 * iflash_mem.hpp
 *
 *  Created on: 04-01-2011
 *      Author: lucck
 */

#ifndef IFLASH_MEM_HPP_
#define IFLASH_MEM_HPP_

/* ------------------------------------------------------------------ */
namespace fnd {
/* ------------------------------------------------------------------ */
class iflash_mem
{
public:
	//Page offset type
	typedef unsigned short poffs_t;
	//Page address type
	typedef unsigned short paddr_t;
	//Page len
	typedef unsigned short pglen_t;
	//Read one element from flash return bytes written or error
	virtual int read_halfword(paddr_t pg, poffs_t pa, unsigned short &val) const = 0;
	//Write one lement to flash return bytes written or error
	virtual int write_halfword(paddr_t pg, poffs_t pa, unsigned short val) = 0;
	//Read one element from flash return bytes written or error
	virtual int read_word(paddr_t pg, poffs_t pa, unsigned &val) const = 0;
	//Write one lement to flash return bytes written or error
	virtual int write_word(paddr_t pg, poffs_t pa, unsigned val) = 0;
	//Erase page return status
	virtual int page_erase(paddr_t pa) = 0;
	//Get pagesize
	virtual poffs_t page_size() const = 0;
	//Get numpages
	virtual paddr_t num_pages() const = 0;
};

/* ------------------------------------------------------------------ */
}
/* ------------------------------------------------------------------ */
#endif /* IFLASH_MEM_HPP_ */
