/* 
    LCFMETER - Firmware for measure L, C, F using STM32F100 device
    Copyright (C) 2012  Lucjan Bryndza <lucck@boff.pl>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>. 
*/

/* ------------------------------------------------------------------ */
/*
 * virtual_eeprom.hpp
 *
 *  Created on: 04-01-2011
 *      Author: lucck
 */
/* ------------------------------------------------------------------ */
#ifndef VIRTUAL_EEPROM_HPP_
#define VIRTUAL_EEPROM_HPP_

#include "iflash_mem.hpp"
/* ------------------------------------------------------------------ */
namespace fnd
{

/* ------------------------------------------------------------------ */
class virtual_eeprom
{
public:
	static const unsigned value_size = sizeof(unsigned);
	enum errno
	{
		ERRNO_OK,
		ERRNO_INIT = -7000,
		ERRNO_VA_RANGE = -7001,
		ERRNO_NO_SPACE = -7002,
		ERRNO_NOT_FOUND = -7003
	};
	virtual_eeprom(iflash_mem &_flash, unsigned flash_sector=0);
	int read(unsigned address, unsigned &value ) const;
	int write(unsigned address, unsigned value);
	unsigned get_max_index() const { return va_max; }
protected:
	virtual void lock() const {}
	virtual void unlock() const {}
private:
	int find_free_slot() const;
	int sort_flash();
private:
	iflash_mem &flash;
	iflash_mem::paddr_t active_page;
	iflash_mem::paddr_t inactive_page;
	const unsigned va_max;
};
/* ------------------------------------------------------------------ */
}
/* ------------------------------------------------------------------ */
#endif /* VIRTUAL_EEPROM_HPP_ */
