/* 
    LCFMETER - Firmware for measure L, C, F using STM32F100 device
    Copyright (C) 2012  Lucjan Bryndza <lucck@boff.pl>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>. 
*/

/*
 * stm32dma.h
 *
 *  Created on: 09-03-2011
 *      Author: lucck
 */

#ifndef STM32DMA_H_
#define STM32DMA_H_
/* ---------------------------------------------------------------------------- */
#include <stm32f10x_lib.h>
#include <cstddef>
/* ---------------------------------------------------------------------------- */

#ifdef __cplusplus
 namespace stm32 {
#endif

/* ---------------------------------------------------------------------------- */
enum dma_cntr
{
	DMACNTR_1,
	DMACNTR_2
};


/* ---------------------------------------------------------------------------- */
//Enable the selected dma controller
static inline void dma_enable(enum dma_cntr ch)
{
	if(ch == DMACNTR_1)
	{
		RCC->AHBENR |= RCC_AHBENR_DMA1EN;
		DMA1->IFCR = 0xFFFFFFFF;
	}
#if defined(STM32F10X_HD) || defined(STM32F10X_CL)
	else if(ch == DMACNTR_2)
	{
		RCC->AHBENR |= RCC_AHBENR_DMA2EN;
		DMA2->IFCR = 0xFFFFFFFF;
	}
#endif
}
/* ---------------------------------------------------------------------------- */
//Disable the dma channel
void dma_disable(enum dma_cntr ch)
{
	if(ch == DMACNTR_1)
	{
		RCC->AHBENR &= ~RCC_AHBENR_DMA1EN;
	}
#if defined(STM32F10X_HD) || defined(STM32F10X_CL)
	else if(ch == DMACNTR_2)
	{
		RCC->AHBENR &= ~RCC_AHBENR_DMA2EN;
	}
#endif
}
/* ---------------------------------------------------------------------------- */
//Dma channel enable
static inline void dma_channel_config(DMA_Channel_TypeDef *DMAchx, uint32_t config,
		volatile void * mem, volatile void *periph, size_t size)
{
	DMAchx->CCR = config;
	DMAchx->CPAR = (unsigned long)periph;
	DMAchx->CMAR = (unsigned long)mem;
	DMAchx->CNDTR = size;
}
/* ---------------------------------------------------------------------------- */
static inline void dma_channel_enable(DMA_Channel_TypeDef *DMAchx)
{
	DMAchx->CCR |= 0x1;
}
/* ---------------------------------------------------------------------------- */
static inline void dma_channel_disable(DMA_Channel_TypeDef *DMAchx)
{
	DMAchx->CCR &= ~0x1;
}
/* ---------------------------------------------------------------------------- */
static inline void dma_irq_enable( DMA_Channel_TypeDef* DMAchx, unsigned it)
{
	DMAchx->CCR |= it;
}
/* ---------------------------------------------------------------------------- */
static inline void dma_irq_disable( DMA_Channel_TypeDef* DMAchx, unsigned it)
{
	DMAchx->CCR &= ~it;
}
/* ---------------------------------------------------------------------------- */
static inline void dma_clear_flag(uint32_t flag)
{
  if ((flag & 0x10000000) != 0)
  {
    /* Clear the selected DMA flags */
    DMA2->IFCR = flag;
  }
  else
  {
    /* Clear the selected DMA flags */
    DMA1->IFCR = flag;
  }
}
/* ---------------------------------------------------------------------------- */

#ifdef __cplusplus
}
#endif
/* ---------------------------------------------------------------------------- */
#endif /* STM32DMA_H_ */
