/* 
    LCFMETER - Firmware for measure L, C, F using STM32F100 device
    Copyright (C) 2012  Lucjan Bryndza <lucck@boff.pl>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>. 
*/

/* ------------------------------------------------------------------ */
/*
 * stm32fmc.hpp
 *
 *  Created on: 04-01-2011
 *      Author: lucck
 */
/* ------------------------------------------------------------------ */
#ifndef STM32FMC_HPP_
#define STM32FMC_HPP_
/* ------------------------------------------------------------------ */
#include "iflash_mem.hpp"

/* ------------------------------------------------------------------ */
namespace stm32
{
/* ------------------------------------------------------------------ */
class stm32fmc: public fnd::iflash_mem
{
public:
	enum errno
	{
	  ERRNO_COMPLETE = 0,
	  ERRNO_BUSY = -1,
	  ERRNO_PG = -2,
	  ERRNO_WRP = -3,
	  ERRNO_TIMEOUT = -4
	};
public:
	//Default constructor
	stm32fmc();
	//Virtual destructor
	virtual ~stm32fmc();
	//Get pagesize
	virtual poffs_t page_size() const;
	//Get numpages
	virtual paddr_t num_pages() const;
	//Erase page return status
	virtual int page_erase(paddr_t pa);
	//Write half word into flash
	virtual int write_halfword(paddr_t pg, poffs_t pa, unsigned short val);
	//Read halfword
	virtual int read_halfword(paddr_t pg, poffs_t pa, unsigned short &val) const;
	//Read one element from flash return bytes written or error
	virtual int read_word(paddr_t pg, poffs_t pa, unsigned &val) const;
	//Write one lement to flash return bytes written or error
	virtual int write_word(paddr_t pg, poffs_t pa, unsigned val);
private:
	errno wait_for_last_operation(unsigned timeout);
	errno get_status() const;
};
/* ------------------------------------------------------------------ */
}
/* ------------------------------------------------------------------ */
#endif /* STM32FMC_HPP_ */
/* ------------------------------------------------------------------ */
