/* 
    LCFMETER - Firmware for measure L, C, F using STM32F100 device
    Copyright (C) 2012  Lucjan Bryndza <lucck@boff.pl>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>. 
*/

/*
 * =====================================================================================
 *
 *       Filename:  frequency_measure.hpp
 *
 *    Description:  Class for automatic measure frequency
 *
 *
 *        Version:  1.0
 *        Created:  31.01.2012 13:24:53
 *       Revision:  none
 *       Compiler:  gcc
 *
 *         Author:  Lucjan Bryndza (lb), lucck@boff.pl
 *   Organization:
 *
 * =====================================================================================
 */
/* ------------------------------------------------------------------ */
#ifndef FREQENCY_MEASURE_HPP_
#define FREQENCY_MEASURE_HPP_
/* ------------------------------------------------------------------ */
#include "imeasure.hpp"
#include "tim_freq_measure.hpp"
#include <cstdlib>
/* ------------------------------------------------------------------ */
namespace app {
/* ------------------------------------------------------------------ */
//Class for frequency measure
class freqency_measure: public app::i_measure
{
private:
	//Maximum low range
	static const long MAX_ALLOWED_IDLE_DIFF = 128 * 1024;
	//Hw divider
	static const unsigned long HW_DIVIDE = 64UL;
public:
	//Enum periph mode
	enum periph_mode { mode_invalid = 256 ,mode_direct , mode_divide_64 };
	//Constructor
	freqency_measure();
	//Destructor
	~freqency_measure();
	//Get calculated value
	virtual int operator()( real_t &mvalue);
	//Enable or disable perhipheral
	virtual void enable( bool en );
	//Set device in selected mode
	virtual void set_mode( int mode );
	//Device is in idle state can sleep
	virtual bool is_idle() const
	{
		return m_idle;
	}
	virtual int get_mode() const
	{
		return m_mode;
	}
private:
	void update_idle_state( long  f )
	{
		m_idle = (f==0 || (std::abs(m_prev_freq - f)>MAX_ALLOWED_IDLE_DIFF) );
		m_prev_freq = f;
	}
private:
	//Frequency measure tool
	dev::tim_freq_measure m_freq_meter;
	//Current mode
	periph_mode m_mode;
	//Previous freqency
	long m_prev_freq;
	//Idle state
	bool m_idle;
};
/* ------------------------------------------------------------------ */
} /* namespace app */
/* ------------------------------------------------------------------ */
#endif /* FREQENCY_MEASURE_HPP_ */
/* ------------------------------------------------------------------ */
