/* 
    LCFMETER - Firmware for measure L, C, F using STM32F100 device
    Copyright (C) 2012  Lucjan Bryndza <lucck@boff.pl>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>. 
*/

/*
 * =====================================================================================
 *
 *       Filename:  i_measure
 *
 *    Description:  Interface for measure value
 *
 *        Version:  1.0
 *        Created:  31.01.2012 13:35:39
 *       Revision:  none
 *       Compiler:  gcc
 *
 *         Author:  Lucjan Bryndza (lb), lucck@boff.pl
 *   Organization:
 *
 * =====================================================================================
 */

/* #####   HEADER FILE INCLUDES   ################################################### */

#ifndef IMEASURE_H_
#define IMEASURE_H_
/* ------------------------------------------------------------------ */
#include <noncopyable.hpp>

/* ------------------------------------------------------------------ */
namespace app {
/* ------------------------------------------------------------------ */
/** Measure interface for device */
/* ------------------------------------------------------------------ */
//Real type used for floating point
typedef double real_t;

/* ------------------------------------------------------------------ */

class i_measure : private fnd::noncopyable
{
//Special values from result
public:
	/* No result for example open terminals in L mode, or
	 * short terminals in the C mode
	 */
	enum result
	{
		RESULT_OK,					/* Status is ok */
		RESULT_INVALID,				/* Result invalid for example open terminals in L mode */
		RESULT_NO_NEWDATA,			/* New measure NA */
		RESULT_NO_CALIBRATION,		/* Device not calibrated */
        RESULT_INVALID_CALIBRATION, /* Invalid calibration for example open nodes */
		RESULT_CALIBRATE_0PERCENT,	/* Calibration pos 0 100% */
		RESULT_CALIBRATE_100PERCENT = RESULT_CALIBRATE_0PERCENT + 100
	};
	static const int NOT_IMPLEMENTED = -1;
public:
	//Device is in idle state can sleep
	virtual bool is_idle() const = 0;
	//Get calculated value
	virtual int operator()(real_t &value) = 0;
	//Enable or disable perhipheral
	virtual void enable( bool en ) = 0;
	//Enable or disable perhipheral
	virtual void set_mode( int mode ) = 0;
	//Get mode
	virtual int get_mode() const = 0;
};
/* ------------------------------------------------------------------ */
} /* namespace app */

/* ------------------------------------------------------------------ */
#endif /* IMEASURE_H_ */
/* ------------------------------------------------------------------ */
