/* 
    LCFMETER - Firmware for measure L, C, F using STM32F100 device
    Copyright (C) 2012  Lucjan Bryndza <lucck@boff.pl>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>. 
*/

/*
 * keyboard.hpp
 *
 *  Created on: 26-01-2012
 *      Author: lucck
 */
/* ------------------------------------------------------------------ */
#ifndef KEYBOARD_HPP_
#define KEYBOARD_HPP_
/* ------------------------------------------------------------------ */
#include <noncopyable.hpp>
#include <stdint.h>
/* ------------------------------------------------------------------ */
namespace dev
{
/* ------------------------------------------------------------------ */
namespace tim {
namespace _internal {

	void keyboard_tick_handler();
}}
/* ------------------------------------------------------------------ */
class keyboard : public fnd::noncopyable
{
	friend void tim::_internal::keyboard_tick_handler();
private:   //Class configuration data
	static const unsigned REPEAT_TIME = 5;
	static const unsigned DELAY_TIME = 15;
	static const unsigned NUM_KEYS = 2;
	static const unsigned NUM_CYCLES = 7;
	static const unsigned NUM_ONES =   6;
	typedef uint8_t key_val_t;
	typedef uint8_t bitset_t;
public:
	//Key definition
	enum keys_def
	{
		KEY_NONE = 0,
		KEY_ON_MODE = 1,
		KEY_OFF = 2,
		KEY_RPT = 0x80
	};
	//keyboard constructor
	keyboard();
	//keyboyard destructor
	~keyboard();
	//Get keys
	int get() const;
	//Wait for all keys will be released
	int wait_for_release_all() const;
private:
	//On time event called from ISR content
	void on_time_event();
	void got_key( key_val_t new_key );
private:
	mutable volatile key_val_t key;
	volatile key_val_t key_back;
	volatile key_val_t last_key;
	volatile key_val_t repeat_k;
	volatile key_val_t delay_k;
	volatile key_val_t bitset_cycle;
	volatile bitset_t keys_bitset[NUM_KEYS];
};
/* ------------------------------------------------------------------ */
} /* namespace app */
/* ------------------------------------------------------------------ */
#endif /* KEYBOARD_HPP_ */
/* ------------------------------------------------------------------ */
