/* 
    LCFMETER - Firmware for measure L, C, F using STM32F100 device
    Copyright (C) 2012  Lucjan Bryndza <lucck@boff.pl>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>. 
*/

/*
 * =====================================================================================
 *
 *       Filename:  lc_measure.hpp
 *
 *    Description:   Class for automatic measure L or C
 *
 *
 *        Version:  1.0
 *        Created:  31.01.2012 13:24:53
 *       Revision:  none
 *       Compiler:  gcc
 *
 *         Author:  Lucjan Bryndza (lb), lucck@boff.pl
 *   Organization:
 *
 * ===================================================================================== */

#ifndef LC_MEASURE_HPP_
#define LC_MEASURE_HPP_
/* ------------------------------------------------------------------ */
#include "imeasure.hpp"
#include "tim_freq_measure.hpp"
#include <cstdlib>
/* ------------------------------------------------------------------ */
namespace app {

/* ------------------------------------------------------------------ */
/* Class for measure LC parameters */
class lc_measure : public app::i_measure
{
public:
	//Enum periph mode
	enum periph_mode {
		mode_invalid = 1024 ,mode_L , mode_C, mode_calibrate_L, mode_calibrate_C
	};
	static const int Cref_int_min = 90000;
	static const int Cref_int_max = 1100000;
	//Constructor
	lc_measure();
	//Destructor
	~lc_measure();
	//Get calculated value
	virtual int operator()( real_t &mvalue );
	//Enable or disable perhipheral
	virtual void enable( bool en );
	//Set device in selected mode
	virtual void set_mode( int mode );
	//Get mode
	//Get current mode
	virtual int get_mode() const
	{
		return m_mode;
	}
	//Get idle mode
	virtual bool is_idle() const
	{
		return m_idle;
	}
	//Get cref value
	real_t get_Cref_value() const;
private:
	enum device_mode {  dmode_L, dmode_C, dmode_C_Cref , dmode_L_Cref };
	enum device_state { s_switch_C,
		s_wait_C, s_measure_C, s_wait_CC, s_measure_CC ,s_ready };
	static const unsigned C_invalid_freq_value = 1000;
    //Maximum range
	static const long C_min_freq_diff = 256;
	//Switch L, C, C+Cref
	void switch_LCC( device_mode dev);
	//Update idle state
	void update_idle_state( long f )
	{
		m_idle = (f==0 || (std::abs( f - static_cast<long>(m_prev_freq)) < C_min_freq_diff ) );
	}
private:
	//Frequency measure tool
	dev::tim_freq_measure m_freq_meter;
	//Previous frequency for scale or num cycles
	short m_num_cycles;
	unsigned long m_prev_freq;
	//Current mode
	periph_mode m_mode;
	//device state
	device_state m_state;
	//Zero values
	unsigned long m_f1_L;		//Only LC
	unsigned long m_f2_L;		//LC + Ccal
	//Zero values
	unsigned long m_f1_C;		//Only LC
	unsigned long m_f2_C;		//LC + Ccal
	//Temporary calibration values
	unsigned long m_tmp_f1;
	//Idle state
	bool m_idle;
};
/* ------------------------------------------------------------------ */
} /* namespace app */
/* ------------------------------------------------------------------ */
#endif /* LC_MEASURE_HPP_ */
/* ------------------------------------------------------------------ */
