/* 
    LCFMETER - Firmware for measure L, C, F using STM32F100 device
    Copyright (C) 2012  Lucjan Bryndza <lucck@boff.pl>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>. 
*/

/*
 * example1.c
 *
 *  Created on: 18-09-2010
 *      Author: lucck
 */
/* ------------------------------------------------------------------ */
#include <stm32f10x_lib.h>
#include <stm32pwr.h>
#include <stm32exti.h>
#include <stm32rcc.h>
#include <system.h>
#include "config.h"
/* ------------------------------------------------------------------ */
//! Configure ADC prescaler to 8
#define RCC_CFGR_ADCPRE_2 (0<<14)
#define RCC_CFGR_ADCPRE_4 (1<<14)
#define RCC_CFGR_ADCPRE_6 (2<<14)
#define RCC_CFGR_ADCPRE_8 (3<<14)
//! HSE oscilator control
#define RCC_CR_HSI_ON  (1<<0)
//SysTimer values
#define MHZ 1000000
#define IWDG_RELOAD_VALUE 78 //1.2s
/* ------------------------------------------------------------------ */
void _external_exit(void);

/* ------------------------------------------------------------------ */
/** Cortex stm32 System setup
 * Clock and flash configuration for selected rate
 */
void uc_periph_setup()
{
	//Configure CLK clock
    RCC->CR &= ~RCC_CR_HSEON;
    //Disable Bypass
    RCC->CR &= ~RCC_CR_HSEBYP;
    //Enable high speed oscilator
    RCC->CR  |= RCC_CR_HSEON;
    //Wait for setup HSE
    while(1)
    {
        if(RCC->CR & RCC_CR_HSERDY) break;
    }
#if !defined (STM32F10X_LD_VL) && !defined (STM32F10X_MD_VL) && !defined (STM32F10X_HD_VL)
    //Configure flash: Prefetch enable and 1 wait state
    FLASH->ACR = FLASH_ACR_PRFTBE | FLASH_ACR_LATENCY_2;
#endif
    /* Configure system clocks ALL clocks freq 72MHZ APB1 36MHz
    * PLL x 2
     */
    RCC->CFGR = RCC_CFGR_ADCPRE_2  | RCC_CFGR_PPRE1_DIV1 | RCC_PLLMul_2 |
    		RCC_CFGR_PPRE2_DIV1 | RCC_CFGR_PLLSRC;
#ifdef  STM32F10X_CL
    RCC->CFGR2 = RCC_CFGR2_PREDIV1_DIV3;
#endif
    // At end disable HSI oscilator for power reduction
    RCC->CR &= ~RCC_CR_HSI_ON;
    //Enable PLL
    RCC->CR |= RCC_CR_PLLON;

    //Wait for PLL sync
    while(1)
    {
	    if(RCC->CR & RCC_CR_PLLRDY)
	    {
	        //PLL as system clock
	        RCC->CFGR |=  RCC_CFGR_SW_PLL;
	        break;
	    }
    }
    //Setup NVIC vector at begin of flash
    SCB->VTOR = NVIC_VectTab_FLASH;
}

/* ------------------------------------------------------------------ */
//! This function is called just before call global constructors
void _external_startup(void)
{
#ifndef PDEBUG
	//If it is not software reset exit
	if(!rcc_get_flag_status( RCC_FLAG_SFTRST ) )
	{
		rcc_clear_flag();
		_external_exit();
	}
	//If it isnt a watchdog reset exit
	if( rcc_get_flag_status( RCC_FLAG_IWDGRST ) )
	{
		rcc_clear_flag();
		_external_exit();
	}
	//Clear rcc flags
	rcc_clear_flag();
#endif
	//Initialize system perhipheral
	uc_periph_setup();
	//Initialize heap
	//tiny_alloc_init();

	//1 bit for preemtion priority
	nvic_priority_group(NVIC_PriorityGroup_1);

	//Set timer priority
	nvic_set_priority(SysTick_IRQn,1,0x7);

	//Setup the systick timer
	systick_config( HZ * (HCLK_HZ/(8*MHZ)) );

	//Used gpio port setup
	rcc_periph_clock_cmd( rcc_clk_apb2, RCC_APB2Periph_GPIOA |RCC_APB2Periph_GPIOB | RCC_APB2Periph_GPIOC | RCC_APB2Periph_GPIOD, true );

	//Standby domain
	rcc_periph_clock_cmd( rcc_clk_apb1, RCC_APB1Periph_PWR, true );
#ifndef PDEBUG
	iwdt_setup( IWDG_Prescaler_256,  IWDG_RELOAD_VALUE );
#endif
}
/* ------------------------------------------------------------------ */
/** Called after destructor call
 * Goto sleep mode after complete
 */
void _external_exit(void)
{
	//Used gpio port setup
	rcc_periph_clock_cmd( rcc_clk_apb2,  RCC_APB2Periph_GPIOB | RCC_APB2Periph_GPIOC | RCC_APB2Periph_GPIOD, false );
	/* Configure EXTI controller for generate event */
	rcc_periph_clock_cmd( rcc_clk_apb2, RCC_APB2Periph_AFIO, true );
	rcc_periph_clock_cmd( rcc_clk_apb1, RCC_APB1Periph_PWR, true );
	gpio_exti_line_config( GPIO_PortSourceGPIOA, GPIO_PinSource1 );
	exti_init( EXTI_Line1, EXTI_Mode_Event, EXTI_Trigger_Rising, true );
	pwr_enter_stop_mode( PWR_Regulator_LowPower, PWR_STOPEntry_WFE );
	nvic_system_reset();

}
/* ------------------------------------------------------------------ */
