/* 
    LCFMETER - Firmware for measure L, C, F using STM32F100 device
    Copyright (C) 2012  Lucjan Bryndza <lucck@boff.pl>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>. 
*/

/*
 * stepup_converter.hpp
 *
 *  Created on: 21-01-2012
 *      Author: lucck
 */

#ifndef STEPUP_CONVERTER_HPP_
#define STEPUP_CONVERTER_HPP_
/* ------------------------------------------------------------------ */
#include <noncopyable.hpp>
#include <stdint.h>
/* ------------------------------------------------------------------ */
namespace dev
{
/* ------------------------------------------------------------------ */
namespace _internal {
extern "C" {
	void dma1_channel1_isr_vector(void) __attribute__((interrupt));
}}

namespace tim {
namespace _internal {

	void stepup_converter_tick_handler();
}}

/* ------------------------------------------------------------------ */
class stepup_converter : private fnd::noncopyable
{
	friend void _internal::dma1_channel1_isr_vector(void);
	friend void tim::_internal::stepup_converter_tick_handler();
public:
	//Get vbat in mV
	int get_Vbat() const
	{
		return (ADC_MAXVAL * VREF_INT_MV)/m_adc_regs[VREF_POSTAB];
	}
	//Get vzas in mV
	int get_Vzas() const
	{
		return (((m_adc_regs[VIN_POSTAB]*VREF_INT_MV)/
			m_adc_regs[VREF_POSTAB] ) * (64*(R1_DIV_VAL+R2_DIV_VAL)/R1_DIV_VAL))/64;
	}
	//Constructor
	stepup_converter();
	//Destructor
	~stepup_converter();
private:
	//Start conversion
	inline void start_conv();
	//Calculate PWM output
	inline void calculate_pwm_out();
	//Voltage to pwm value
	static inline int v2pwm(int v);
	//IRQ priority
	static const int IRQ_PRIO = 1;
	static const int IRQ_SUBPRIO = 7;
	//Table
	enum { VIN_POSTAB, VREF_POSTAB };
	static const int STACK_DEPTH = 512;
	static const int TASK_PRIO = 0;
	static const int VREF_INT_MV = 1200;
	static const int ADC_MAXVAL = 4095;
	static const int R1_DIV_VAL = 2700;
	static const int R2_DIV_VAL = 10000;
private:
	uint16_t m_adc_regs[2];					//ADC data values
	volatile int m_pwm_value;				//Pwm value
};
/* ------------------------------------------------------------------ */
} /* namespace app2 */
#endif /* STEPUP_CONVERTER_HPP_ */
/* ------------------------------------------------------------------ */
