/* 
    LCFMETER - Firmware for measure L, C, F using STM32F100 device
    Copyright (C) 2012  Lucjan Bryndza <lucck@boff.pl>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>. 
*/

/*
 * sys_time.hpp
 *
 *  Created on: 22-01-2012
 *      Author: lucck
 */
/* ------------------------------------------------------------------ */
#ifndef SYS_TIME_HPP_
#define SYS_TIME_HPP_
/* ------------------------------------------------------------------ */
#include <system.h>
/* ------------------------------------------------------------------ */
namespace dev {
namespace tim {

	//tick def
	typedef unsigned int tick_t;
	//Read tick
	tick_t read_tick();
	//Check if timer elapsed
	inline int timer_elapsed(tick_t t1, tick_t timeout)
	{
	    tick_t t2 = read_tick();
	    if( t2 >= t1)   //Not overflow
	        return t2 - t1 > timeout;
	    else           //Overflow
	        return t1 - t2 > timeout;
	}
	//Busy wait selected time
	inline void timer_wait(tick_t timeout)
	{
		tick_t now = read_tick();
		while( !timer_elapsed(now, timeout) )
		{
			stm32::iwdt_reset();
			stm32::wfi();
		}
	}

}}
/* ------------------------------------------------------------------ */
#endif /* SYS_TIME_HPP_ */
/* ------------------------------------------------------------------ */
