/* 
    LCFMETER - Firmware for measure L, C, F using STM32F100 device
    Copyright (C) 2012  Lucjan Bryndza <lucck@boff.pl>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>. 
*/

/*
 * theapplication.h
 *
 *  Created on: 05-01-2012
 *      Author: lucck
 */
/* ------------------------------------------------------------------ */
#ifndef THEAPPLICATION_H_
#define THEAPPLICATION_H_
/* ------------------------------------------------------------------ */
#include "stepup_converter.hpp"
#include "lcddisp.h"
#include "keyboard.hpp"
#include "freqency_measure.hpp"
#include "lc_measure.hpp"
#include "sys_time.hpp"
/* ------------------------------------------------------------------ */
namespace app
{
/* ------------------------------------------------------------------ */
class the_application
{
private:    //Constant defs
	static const unsigned short GATE_TIME = 200;
    static const dev::tim::tick_t C_shutdown_timeout = 10 * 60 * 1000;
public:
	//Constructor
	the_application();
	//Run
	void run();
	//Current mode
	enum e_mode
	{
	    e_mode_first=-1, e_mode_disp_C, e_mode_disp_L, e_mode_disp_F_hi, e_mode_disp_F_low, e_mode_disp_last
	};
private:
	//Display the value
	void display_measured_value(real_t value);
	//Display bottom line (battery level)
	void display_battery_status();
	//Display calibration screen
	void display_calibration_screen( int percent, int value );
	//Go to sleep
	void goto_sleep()
	{
		stm32::iwdt_reset();
		stm32::wfi();
	}
	//On change mode
	void on_change_mode();
	//On start calibration
	void on_start_calibration();
	//Do process data
	void do_process_data( i_measure &measure );
	//Increment mode
	void increment_mode();
	//Decrement mode
	void decrement_mode();
	//Set calibration value
#ifdef ENABLE_SERVICE_CALIBRATION_MENU
	void service_mode_calibration();
#endif
	//About screen
	void about_screen();
private:
	//Stepup converter class
	dev::stepup_converter m_power_conv;
	//Lcd display
    dev::lcd_disp m_lcd;
    //Keyboard
    dev::keyboard m_kbd;
    //Frequency measure
    freqency_measure m_f_meter;
    //LC measure
    lc_measure m_lc_meter;
    //Current mode
    e_mode m_mode;
    //Current display pointer
    i_measure *m_curr_disp;
    //Previous refresh bottom bar
    dev::tim::tick_t m_last_bottom_refresh_time;
    //Last used time
    dev::tim::tick_t m_last_used_time;

};
/* ------------------------------------------------------------------ */
} /* namespace app */
/* ------------------------------------------------------------------ */
#endif /* THEAPPLICATION_H_ */
/* ------------------------------------------------------------------ */
