/* 
    LCFMETER - Firmware for measure L, C, F using STM32F100 device
    Copyright (C) 2012  Lucjan Bryndza <lucck@boff.pl>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>. 
*/

/*
 * =====================================================================================
 *
 *       Filename:  tim_freq_measure.hpp
 *
 *    Description:  timer freq measure base version
 *
 *
 *        Version:  1.0
 *        Created:  31.01.2012 13:24:53
 *       Revision:  none
 *       Compiler:  gcc
 *
 *         Author:  Lucjan Bryndza (lb), lucck@boff.pl
 *   Organization:  
 *
 * =====================================================================================
 */

#ifndef _TIM_FREQ_MEASURE_HPP
#define _TIM_FREQ_MEASURE_HPP

/* #####   HEADER FILE INCLUDES   ################################################### */

#include <noncopyable.hpp>
#include <system.h>
/* ------------------------------------------------------------------ */

namespace dev {

/* ------------------------------------------------------------------ */
namespace _internal {
extern "C" {
	void tim2_isr_vector(void) __attribute__((interrupt));
	void tim3_isr_vector(void) __attribute__((interrupt));
	void tim1_brk_tim15_isr_vector(void) __attribute__((interrupt));
}}

/* ------------------------------------------------------------------ */
class tim_freq_measure : private fnd::noncopyable
{
	friend void _internal::tim2_isr_vector(void);
	friend void _internal::tim3_isr_vector(void);
	friend void _internal::tim1_brk_tim15_isr_vector(void);
private:
	static const int IRQ_PRIO = 0;
	static const int IRQ_SUB =  7;
	//Timer base period for prescaler
	static const unsigned long BASE_PERIOD = 1000;
	static const unsigned long GATE_TIME_MS = 250;
	static const unsigned long SECOND_C = 1000;
	static const unsigned long GATE_NOT_ACTIVE_MS = 5;
private:
	//Handle overflow IRQ
	void handle_overflow();
	//Handle measure finished
	void handle_measure_finished();
public:
	//No measure statement
	static const unsigned long NO_NEW_DATA = 0xFFFFFFFF;
    static const unsigned long RAW_DATA_DIVIDE = SECOND_C/GATE_TIME_MS;
	/** Construct for measure timer
	 * @param tim Used timer
	 */
	explicit tim_freq_measure( TIM_TypeDef * tim );
	//Destructor
	~tim_freq_measure();
	//Frequency
	unsigned long operator()()
	{
		unsigned long value = stm32::atomic_xchg_word( &m_capture, NO_NEW_DATA );
		if( value != NO_NEW_DATA ) value *= (SECOND_C/GATE_TIME_MS);
		return  value;
	}
private:		//Temporary
	TIM_TypeDef * const m_tim;   //Timer pointer
	volatile unsigned short m_hi_cnt;    //Hi counter
	volatile unsigned long  m_capture;
};

/* ------------------------------------------------------------------ */
}		/* -----  end of namespace dev  ----- */
#endif
